;-------------------------------------------------------------------
;Copyright 1989, SHAND, LTD.
;		 Rebecca Lane
;		 Pleasantville, New York 10570
;		 TEL: (914) 747 1272
;		 FAX and Modem: (914) 747 1796
;		 Clarion BBS: ARNS NEUMAN
;		 COMPUSERV: 71250,2421	
;
;	MODULE: DELAY.ASM
;		Specify time delay 
;-------------------------------------------------------------------

;----------SYNTAX and CAVEATS---------------------------------------
;
;	DELAY(hours,minutes,seconds,hundredths)
;
;	All parameters are required so use 0 where necessary.
;
;	DO NOT use any value over 127 for any parameter. 
;		(which should never be necessary)
;	
;	The parameters are treated as signed bytes and they are added to
;       bytes, therefore, if the total is greater than 127 or less 0
;	results become unpredictable. There is no bounds checking done.
;	Best idea is to be reasonable and use values that make sense,
;	            ie: 0 < hours < 23, 0 < minutes < 59, 0 < seconds < 59
;		    and 0 < hundredths < 99.
;	This procedure really only accurate to about 1/20th of a second
;--------------------------------------------------------------------
;	CLARION MAP
;	
;	MODULE('DELAY'), BINARY
;	   PROC(DELAY)
;	.
;--------------------------------------------------------------------
_TEXT	SEGMENT BYTE PUBLIC 'CODE'
_TEXT	ENDS
_DATA	SEGMENT WORD PUBLIC 'DATA'
_DATA	ENDS
_BSS	SEGMENT WORD PUBLIC 'BSS'
_BSS	ENDS
DGROUP	GROUP _TEXT,_DATA,_BSS

_TEXT	SEGMENT BYTE PUBLIC 'CODE'	;code and data in same segment
	ASSUME CS:_TEXT,DS:DGROUP

;---------- BINARY MODULE HEADER
		DB	'BIO'		;Bin signature for version 2.0
		PUBLIC  LIBVEC		;Public vector to header
	LIBVEC	DD	0		;double word reserved
		DW	BINEND		;Length of binary module
		DB	1		;one function in this LEM module

;----------FUNCTION DEFINITION
		DB	'DELAY', 8 DUP(0);Clarion function name padded to 13 
			 		; bytes with last byte 0
		DW	DELAY		;OFFSET WITHIN BINARY MODULE
		DB	0		;0=procedure, 1= function
		DB	4		;4 calling parameters

;----------Parameter 1 (hours) definitions 
		 DB	1		;Parameter is word
       	HOURS    DD	0		
	HOURSL   DW	0		
;----------Parameter 2 (minutes) definitions 
		 DB	1		;Parameter is word
       	MINUTES  DD	0		
	MINUTESL DW	0		
;----------Parameter 3 (seconds) definitions
		 DB	1		;Parameter is word
       	SECONDS  DD	0		
	SECONDSL DW	0		

;----------Parameter 4 (hundredths) definitions
		  DB	1		;Parameter is word
       	HUNDREDS  DD	0		
	HUNDREDSL DW	0		
;----------------------------------------------------------

	PUBLIC	DELAY			;Public procedure 
DELAY 		PROC	FAR		;assembler will generate far return
		MOV	AH,2CH		;DOS function 2C
		INT	21H		;call DOS		
		MOV	AH,CH		;current hours
		MOV	AL,CL		;current minutes
		MOV	BH,DH		;current seconds
		MOV	BL,DL		;current hundredths
		PUSH	BX		;save BX before getting 
					;   parameters
;----------- Get Hours
		LES	BX, HOURS		;get hours and 
		ADD     AH,BYTE PTR ES:[BX]	;add hours to current time

;---------- Get Minutes
		LES	BX, MINUTES		;get minutes
		ADD     AL,BYTE PTR ES:[BX]	;add minutes to current time

;----------- Get Seconds and Hundredths
		LES	BX, SECONDS		;get seconds
		MOV	DH, BYTE PTR ES:[BX]	;hold here until BH restored
                LES	BX, HUNDREDS		;get hundredths
		MOV	DL, BYTE PTR ES:[BX]    ;hold here until BL restored
		POP     BX			;restore secs. and 100's
		ADD	BH, DH			;add seconds to current time
		ADD	BL, DL			;add hundredths to current time
;----------Deal with Overflow

CENTS:	
		CMP	BL,100		;is it > 99 hundredths
		JL	SHORT SECS	;jump if below
		SUB	BL,100		;subtract 100
		INC	BH		;increment seconds

SECS:	
		CMP	BH,60		;is it > 59 seconds
		JL	SHORT MINS	;jump if below
		SUB	BH,60		;subtract 60 seconds
		INC	AL		;add 1 minute

MINS:	
		CMP	AL,60		;is it > 59 minutes
		JL	SHORT HRS	;jump if below
		SUB	AL,60		;subtract 60 minutes
		INC	AH		;add 1 hour

HRS:	
		CMP	AH,24		;is it > 23 hours
		JL	SHORT CHECK_TIME
		SUB	AH, 24

CHECK_TIME:
		PUSH AX
		MOV	AH,2CH
		INT	21H
		POP 	AX		
		CMP	CX,AX		;compare hours and minutes
		JNE	CHECK_TIME	;return if above, cannot be below
		CMP	DX,BX		;test seconds and hundredths
		JB	CHECK_TIME	;not there yet so loop
QUIT:
		RET			;Return to Clarion. 
DELAY	ENDP				;end of procedure
BINEND		DB	0		;mark end of LEM
_TEXT		ENDS			;end code segment
		END			;end assembly